import { reduce, and, equals, pipe, has, toPairs, map, zipObj, is } from 'ramda'
import derefSchema from './derefSchema'

const schemaParser = schema => {

  if (has('properties', schema)) {
    schema = schema.properties
  } else if (has('patternProperties', schema)) {
    schema = schema.patternProperties
  }

  let props = pipe(
      toPairs,
      map(zipObj(['key', 'val']))
  )(schema)

  return reduce((acc, o) => {
    if (and(equals('object', o.val.type), is(Object, o.val.properties))) {
      acc[o.key] = emptySchema(o.val.properties)
    } else if (equals('object', o.val.type)) {
      acc[o.key] = {}
    } else if (equals('array', o.val.type)) {
      acc[o.key] = []
    } else if (equals('string', o.val.type)) {
      acc[o.key] = ''
    } else {
      acc[o.key] = null
    }

    return acc
  }, {}, props)
}

/**
 * Creates an empty JSON object based on a JSON schema.
 *
 * @function
 * @param {object} schema - The JSON Schema
 * @returns {object} obj - An object with the JSON Schema structure
 * @example
 *
 * var schema = {
 *  foo: {
 *    type: "string"
 *  },
 *  bar: {
 *    type: "array"
 *  },
 *  baz: {
 *    type: "object"
 *    properties: {
 *      bam: {
 *        type: "string"
 *      }
 *    }
 *  }
 * }
 *
 * emptySchema(schema) // { foo: "", bar: [], baz: { bam: "" } }
 *
 */
const emptySchema = pipe(derefSchema, schemaParser)

export default emptySchema
